@@ -15,59 +15,164 @@
 """
 __docformat__ = "reStructuredText"
 
-from zope.interface import Interface, Attribute
+from zope.interface import Attribute
+from zope.interface import Interface
 from zope.interface.common.mapping import IEnumerableMapping
+from zope.interface.interfaces import IInterface
 
 
+from zope.schema._bootstrapfields import Bool
+from zope.schema._bootstrapfields import Complex
+from zope.schema._bootstrapfields import Decimal
+from zope.schema._bootstrapfields import Field
+from zope.schema._bootstrapfields import Int
+from zope.schema._bootstrapfields import Integral
+from zope.schema._bootstrapfields import Number
+from zope.schema._bootstrapfields import Object
+from zope.schema._bootstrapfields import Rational
+from zope.schema._bootstrapfields import Real
+from zope.schema._bootstrapfields import Text
+from zope.schema._bootstrapfields import TextLine
+
 # Import from _bootstrapinterfaces only because other packages will expect
 # to find these interfaces here.
-from zope.schema._bootstrapfields import Field
-from zope.schema._bootstrapfields import Text
-from zope.schema._bootstrapfields import TextLine
-from zope.schema._bootstrapfields import Bool
-from zope.schema._bootstrapfields import Int
-from zope.schema._bootstrapinterfaces import StopValidation
-from zope.schema._bootstrapinterfaces import ValidationError
-from zope.schema._bootstrapinterfaces import IFromUnicode
-from zope.schema._bootstrapinterfaces import RequiredMissing
-from zope.schema._bootstrapinterfaces import WrongType
 from zope.schema._bootstrapinterfaces import ConstraintNotSatisfied
+from zope.schema._bootstrapinterfaces import IBeforeObjectAssignedEvent
+from zope.schema._bootstrapinterfaces import IContextAwareDefaultFactory
+from zope.schema._bootstrapinterfaces import IFromBytes
+from zope.schema._bootstrapinterfaces import IFromUnicode
+from zope.schema._bootstrapinterfaces import IValidatable
+from zope.schema._bootstrapinterfaces import InvalidValue
+from zope.schema._bootstrapinterfaces import LenOutOfBounds
 from zope.schema._bootstrapinterfaces import NotAContainer
+from zope.schema._bootstrapinterfaces import NotAnInterface
 from zope.schema._bootstrapinterfaces import NotAnIterator
-from zope.schema._bootstrapinterfaces import TooSmall
+from zope.schema._bootstrapinterfaces import OrderableOutOfBounds
+from zope.schema._bootstrapinterfaces import OutOfBounds
+from zope.schema._bootstrapinterfaces import RequiredMissing
+from zope.schema._bootstrapinterfaces import SchemaNotCorrectlyImplemented
+from zope.schema._bootstrapinterfaces import SchemaNotFullyImplemented
+from zope.schema._bootstrapinterfaces import SchemaNotProvided
+from zope.schema._bootstrapinterfaces import StopValidation
 from zope.schema._bootstrapinterfaces import TooBig
 from zope.schema._bootstrapinterfaces import TooLong
 from zope.schema._bootstrapinterfaces import TooShort
-from zope.schema._bootstrapinterfaces import InvalidValue
-from zope.schema._bootstrapinterfaces import IContextAwareDefaultFactory
+from zope.schema._bootstrapinterfaces import TooSmall
+from zope.schema._bootstrapinterfaces import ValidationError
+from zope.schema._bootstrapinterfaces import WrongContainedType
+from zope.schema._bootstrapinterfaces import WrongType
 
 from zope.schema._compat import PY3
-from zope.schema._compat import u
+
 from zope.schema._messageid import _
 
-
-# pep 8 friendlyness
-StopValidation, ValidationError, IFromUnicode, RequiredMissing, WrongType
-ConstraintNotSatisfied, NotAContainer, NotAnIterator
-TooSmall, TooBig, TooLong, TooShort, InvalidValue, IContextAwareDefaultFactory
+__all__ = [
+    # Exceptions
+    'ConstraintNotSatisfied',
+    'InvalidDottedName',
+    'InvalidId',
+    'InvalidURI',
+    'InvalidValue',
+    'LenOutOfBounds',
+    'NotAContainer',
+    'NotAnInterface',
+    'NotAnIterator',
+    'NotUnique',
+    'OrderableOutOfBounds',
+    'OutOfBounds',
+    'RequiredMissing',
+    'SchemaNotCorrectlyImplemented',
+    'SchemaNotFullyImplemented',
+    'SchemaNotProvided',
+    'StopValidation',
+    'TooBig',
+    'TooLong',
+    'TooShort',
+    'TooSmall',
+    'Unbound',
+    'ValidationError',
+    'WrongContainedType',
+    'WrongType',
 
 
-class WrongContainedType(ValidationError):
-    __doc__ = _("""Wrong contained type""")
+    # Interfaces
+    'IASCII',
+    'IASCIILine',
+    'IAbstractBag',
+    'IAbstractSet',
+    'IBaseVocabulary',
+    'IBeforeObjectAssignedEvent',
+    'IBool',
+    'IBytes',
+    'IBytesLine',
+    'IChoice',
+    'ICollection',
+    'IComplex',
+    'IContainer',
+    'IContextAwareDefaultFactory',
+    'IContextSourceBinder',
+    'IDate',
+    'IDatetime',
+    'IDecimal',
+    'IDict',
+    'IDottedName',
+    'IField',
+    'IFieldEvent',
+    'IFieldUpdatedEvent',
+    'IFloat',
+    'IFromBytes',
+    'IFromUnicode',
+    'IFrozenSet',
+    'IId',
+    'IInt',
+    'IIntegral',
+    'IInterfaceField',
+    'IIterable',
+    'IIterableSource',
+    'IIterableVocabulary',
+    'ILen',
+    'IList',
+    'IMapping',
+    'IMinMax',
+    'IMinMaxLen',
+    'IMutableMapping',
+    'IMutableSequence',
+    'INativeString',
+    'INativeStringLine',
+    'INumber',
+    'IObject',
+    'IOrderable',
+    'IPassword',
+    'IPythonIdentifier',
+    'IRational',
+    'IReal',
+    'ISequence',
+    'ISet',
+    'ISource',
+    'ISourceQueriables',
+    'ISourceText',
+    'ITerm',
+    'IText',
+    'ITextLine',
+    'ITime',
+    'ITimedelta',
+    'ITitledTokenizedTerm',
+    'ITokenizedTerm',
+    'ITreeVocabulary',
+    'ITuple',
+    'IURI',
+    'IUnorderedCollection',
+    'IVocabulary',
+    'IVocabularyFactory',
+    'IVocabularyRegistry',
+    'IVocabularyTokenized',
+]
 
 
 class NotUnique(ValidationError):
     __doc__ = _("""One or more entries of sequence are not unique.""")
 
 
-class SchemaNotFullyImplemented(ValidationError):
-    __doc__ = _("""Schema not fully implemented""")
-
-
-class SchemaNotProvided(ValidationError):
-    __doc__ = _("""Schema not provided""")
-
-
 class InvalidURI(ValidationError):
     __doc__ = _("""The specified URI is not valid.""")
 
@@ -84,7 +189,7 @@ class Unbound(Exception):
     __doc__ = _("""The field is not bound.""")
 
 
-class IField(Interface):
+class IField(IValidatable):
     """Basic Schema Field Interface.
 
     Fields are used for Interface specifications.  They at least provide
@@ -125,14 +230,14 @@ class IField(Interface):
     title = TextLine(
         title=_("Title"),
         description=_("A short summary or label"),
-        default=u(""),
+        default=u"",
         required=False,
         )
 
     description = Text(
         title=_("Description"),
         description=_("A description of the field"),
-        default=u(""),
+        default=u"",
         required=False,
         )
 
@@ -313,10 +418,14 @@ class IText(IMinMaxLen, IIterable, IField):
 
 
 # for things which are of the str type on both Python 2 and 3
-if PY3:  # pragma: no cover
-    INativeString = IText
-else:  # pragma: no cover
-    INativeString = IBytes
+class INativeString(IText if PY3 else IBytes):
+    """
+    A field that always contains the native `str` type.
+
+    .. versionchanged:: 4.9.0
+       This is now a distinct type instead of an alias for either `IText`
+       or `IBytes`, depending on the platform.
+    """
 
 
 class IASCII(INativeString):
@@ -343,18 +452,165 @@ class ITextLine(IText):
     """Field containing a unicode string without newlines."""
 
 
-if PY3:  # pragma: no cover
-    INativeStringLine = ITextLine
-else:  # pragma: no cover
-    INativeStringLine = IBytesLine
+class INativeStringLine(ITextLine if PY3 else IBytesLine):
+    """
+    A field that always contains the native `str` type, without any newlines.
+
+    .. versionchanged:: 4.9.0
+       This is now a distinct type instead of an alias for either `ITextLine`
+       or `IBytesLine`, depending on the platform.
+    """
 
 
 class IPassword(ITextLine):
     "Field containing a unicode string without newlines that is a password."
 
+###
+# Numbers
+###
 
-class IInt(IMinMax, IField):
-    """Field containing an Integer Value."""
+##
+# Abstract numbers
+##
+
+class INumber(IMinMax, IField):
+    """
+    Field containing a generic number: :class:`numbers.Number`.
+
+    .. seealso:: :class:`zope.schema.Number`
+    .. versionadded:: 4.6.0
+    """
+    min = Number(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Number(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Number(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+
+
+class IComplex(INumber):
+    """
+    Field containing a complex number: :class:`numbers.Complex`.
+
+    .. seealso:: :class:`zope.schema.Real`
+    .. versionadded:: 4.6.0
+    """
+    min = Complex(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Complex(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Complex(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+
+
+class IReal(IComplex):
+    """
+    Field containing a real number: :class:`numbers.IReal`.
+
+    .. seealso:: :class:`zope.schema.Real`
+    .. versionadded:: 4.6.0
+    """
+    min = Real(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Real(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Real(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+
+class IRational(IReal):
+    """
+    Field containing a rational number: :class:`numbers.IRational`.
+
+    .. seealso:: :class:`zope.schema.Rational`
+    .. versionadded:: 4.6.0
+    """
+
+    min = Rational(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Rational(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Rational(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+
+class IIntegral(IRational):
+    """
+    Field containing an integral number: class:`numbers.Integral`.
+
+    .. seealso:: :class:`zope.schema.Integral`
+    .. versionadded:: 4.6.0
+    """
+    min = Integral(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Integral(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Integral(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+##
+# Concrete numbers
+##
+
+class IInt(IIntegral):
+    """
+    Field containing exactly the native class :class:`int` (or, on
+    Python 2, ``long``).
+
+    .. seealso:: :class:`zope.schema.Int`
+    """
 
     min = Int(
         title=_("Start of the range"),
@@ -375,16 +631,44 @@ class IInt(IMinMax, IField):
         )
 
 
-class IFloat(IMinMax, IField):
-    """Field containing a Float."""
+class IFloat(IReal):
+    """
+    Field containing exactly the native class :class:`float`.
+
+    :class:`IReal` is a more general interface, allowing all of
+    floats, ints, and fractions.
+
+    .. seealso:: :class:`zope.schema.Float`
+    """
 
 
-class IDecimal(IMinMax, IField):
-    """Field containing a Decimal."""
+class IDecimal(INumber):
+    """Field containing a :class:`decimal.Decimal`"""
 
+    min = Decimal(
+        title=_("Start of the range"),
+        required=False,
+        default=None
+        )
+
+    max = Decimal(
+        title=_("End of the range (including the value itself)"),
+        required=False,
+        default=None
+        )
+
+    default = Decimal(
+        title=_("Default Value"),
+        description=_("""The field default value may be None or a legal
+                        field value""")
+        )
+
+###
+# End numbers
+###
 
 class IDatetime(IMinMax, IField):
-    """Field containing a DateTime."""
+    """Field containing a datetime."""
 
 
 class IDate(IMinMax, IField):
@@ -445,6 +729,14 @@ class IDottedName(INativeStringLine):
         )
 
 
+class IPythonIdentifier(INativeStringLine):
+    """
+    A single Python identifier, such as a  variable name.
+
+    .. versionadded:: 4.9.0
+    """
+
+
 class IChoice(IField):
     """Field whose value is contained in a predefined set
 
@@ -469,14 +761,14 @@ class IChoice(IField):
 
 # Abstract
 
-
 class ICollection(IMinMaxLen, IIterable, IContainer):
     """Abstract interface containing a collection value.
 
     The Value must be iterable and may have a min_length/max_length.
     """
 
-    value_type = Field(
+    value_type = Object(
+        IField,
         title=_("Value Type"),
         description=_("Field value items must conform to the given type, "
                       "expressed via a Field."))
@@ -492,6 +784,14 @@ class ISequence(ICollection):
     """Abstract interface specifying that the value is ordered"""
 
 
+class IMutableSequence(ISequence):
+    """
+    Abstract interface specifying that the value is ordered and
+    mutable.
+
+    .. versionadded:: 4.6.0
+    """
+
 class IUnorderedCollection(ICollection):
     """Abstract interface specifying that the value cannot be ordered"""
 
@@ -499,14 +799,14 @@ class IUnorderedCollection(ICollection):
 class IAbstractSet(IUnorderedCollection):
     """An unordered collection of unique values."""
 
-    unique = Attribute("This ICollection interface attribute must be True")
+    unique = Bool(description="This ICollection interface attribute must be True")
 
 
 class IAbstractBag(IUnorderedCollection):
     """An unordered collection of values, with no limitations on whether
     members are unique"""
 
-    unique = Attribute("This ICollection interface attribute must be False")
+    unique = Bool(description="This ICollection interface attribute must be False")
 
 
 # Concrete
@@ -517,7 +817,7 @@ class ITuple(ISequence):
     Python tuple."""
 
 
-class IList(ISequence):
+class IList(IMutableSequence):
     """Field containing a value that implements the API of a conventional
     Python list."""
 
@@ -535,49 +835,56 @@ class IFrozenSet(IAbstractSet):
 
 
 class IObject(IField):
-    """Field containing an Object value."""
+    """
+    Field containing an Object value.
 
-    schema = Attribute(
-        "schema",
-        _("The Interface that defines the Fields comprising the Object.")
+    .. versionchanged:: 4.6.0
+       Add the *validate_invariants* attribute.
+    """
+
+    schema = Object(
+        IInterface,
+        description=_("The Interface that defines the Fields comprising the Object.")
+    )
+
+    validate_invariants = Bool(
+        title=_("Validate Invariants"),
+        description=_("A boolean that says whether ``schema.validateInvariants`` "
+                      "is called from ``self.validate()``. The default is true."),
+        default=True,
     )
 
 
-class IBeforeObjectAssignedEvent(Interface):
-    """An object is going to be assigned to an attribute on another object.
 
-    Subscribers to this event can change the object on this event to change
-    what object is going to be assigned. This is useful, e.g. for wrapping
-    or replacing objects before they get assigned to conform to application
-    policy.
+class IMapping(IMinMaxLen, IIterable, IContainer):
     """
+    Field containing an instance of :class:`collections.Mapping`.
 
-    object = Attribute("The object that is going to be assigned.")
-
-    name = Attribute("The name of the attribute under which the object "
-                     "will be assigned.")
-
-    context = Attribute("The context object where the object will be "
-                        "assigned to.")
-
-
-class IDict(IMinMaxLen, IIterable, IContainer):
-    """Field containing a conventional dict.
-
-    The key_type and value_type fields allow specification
+    The *key_type* and *value_type* fields allow specification
     of restrictions for keys and values contained in the dict.
+
+    """
+    key_type = Object(
+        IField,
+        description=_("Field keys must conform to the given type, expressed via a Field.")
+    )
+
+    value_type = Object(
+        IField,
+        description=_("Field values must conform to the given type, expressed "
+                      "via a Field.")
+    )
+
+
+class IMutableMapping(IMapping):
+    """
+    Field containing an instance of :class:`collections.MutableMapping`.
     """
 
-    key_type = Attribute(
-        "key_type",
-        _("Field keys must conform to the given type, expressed via a Field.")
-    )
 
-    value_type = Attribute(
-        "value_type",
-        _("Field values must conform to the given type, expressed "
-          "via a Field.")
-    )
+class IDict(IMutableMapping):
+    """Field containing a conventional dict.
+    """
 
 
 class ITerm(Interface):
@@ -592,13 +899,14 @@ class ITokenizedTerm(ITerm):
     """
 
     # Should be a ``zope.schema.ASCIILine``, but `ASCIILine` is not a bootstrap
-    # field.
+    # field. `ASCIILine` is a type of NativeString.
     token = Attribute(
         "token",
         """Token which can be used to represent the value on a stream.
 
-        The value of this attribute must be a non-empty 7-bit string.
-        Control characters are not allowed.
+        The value of this attribute must be a non-empty 7-bit native string
+        (i.e., the ``str`` type on both Python 2 and 3).
+        Control characters, including newline, are not allowed.
         """)
 
 
@@ -726,28 +1034,55 @@ class ITreeVocabulary(IVocabularyTokenized, IEnumerableMapping):
 
 
 class IVocabularyRegistry(Interface):
-    """Registry that provides IBaseVocabulary objects for specific fields.
+    """
+    Registry that provides `IBaseVocabulary` objects for specific
+    fields.
+
+    The fields of this package use the vocabulary registry that is
+    returned from :func:`~.getVocabularyRegistry`. This is a hook
+    function; by default it returns an instance of
+    :class:`~.VocabularyRegistry`, but the function
+    :func:`~.setVocabularyRegistry` can be used to change this.
+
+    In particular, the package `zope.vocabularyregistry
+    <https://pypi.org/project/zope.vocabularyregistry/>`_ can be used
+    to install a vocabulary registry that uses the :mod:`zope.component`
+    architecture.
     """
 
-    def get(object, name):
-        """Return the vocabulary named 'name' for the content object
-        'object'.
+    def get(context, name):
+        """
+        Return the vocabulary named *name* for the content object
+        *context*.
 
-        When the vocabulary cannot be found, LookupError is raised.
+        When the vocabulary cannot be found, `LookupError` is raised.
         """
 
 
 class IVocabularyFactory(Interface):
-    """Can create vocabularies."""
+    """
+    An object that can create `IBaseVocabulary`.
+
+    Objects that implement this interface can be registered with the
+    default :class:`~.VocabularyRegistry` provided by this package.
+
+    Alternatively, `zope.vocabularyregistry
+    <https://pypi.org/project/zope.vocabularyregistry/>`_ can be used
+    to install a `IVocabularyRegistry` that looks for named utilities
+    using :func:`zope.component.getUtility` which provide this
+    interface.
+    """
 
     def __call__(context):
-        """The context provides a location that the vocabulary can make use of.
+        """The *context* provides a location that the vocabulary can make use of.
         """
 
 
 class IFieldEvent(Interface):
 
-    field = Attribute("The field that has been changed")
+    field = Object(
+        IField,
+        description="The field that has been changed")
 
     object = Attribute("The object containing the field")
 
